Package: SparseArray
Title: Efficient in-memory representation of multidimensional sparse
        arrays
Description: The SparseArray package is an infrastructure package that
	provides an array-like container for efficient in-memory representation
	of multidimensional sparse data in R. The package defines the
	SparseArray virtual class and two concrete subclasses: COO_SparseArray
	and SVT_SparseArray. Each subclass uses its own internal representation
	of the nonzero multidimensional data, the "COO layout" and the "SVT
	layout", respectively. SVT_SparseArray objects mimic as much as
	possible the behavior of ordinary matrix and array objects in base R.
	In particular, they suppport most of the "standard matrix and array API"
	defined in base R and in the matrixStats package from CRAN.
biocViews: Infrastructure, DataRepresentation
URL: https://bioconductor.org/packages/SparseArray
BugReports: https://github.com/Bioconductor/SparseArray/issues
Version: 1.2.2
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("Vince", "Carey", role="fnd",
		email="stvjc@channing.harvard.edu"),
	person("Rafael A.", "Irizarry", role="fnd",
		email="rafa@ds.harvard.edu"))
Depends: R (>= 4.3.0), methods, Matrix, BiocGenerics (>= 0.43.1),
        MatrixGenerics (>= 1.11.1), S4Vectors, S4Arrays (>= 1.1.6)
Imports: stats, matrixStats, IRanges, XVector
LinkingTo: S4Vectors, IRanges, XVector
Suggests: DelayedArray, testthat, knitr, rmarkdown, BiocStyle
VignetteBuilder: knitr
Collate: sparseMatrix-utils.R SparseArray-class.R
        COO_SparseArray-class.R SVT_SparseArray-class.R
        extract_sparse_array.R read_block_as_sparse.R
        SparseArray-dim-tuning.R SparseArray-aperm.R
        SparseArray-subsetting.R SparseArray-subassignment.R
        SparseArray-abind.R SparseArray-summarization.R
        SparseArray-Ops-methods.R SparseArray-Math-methods.R
        SparseArray-Complex-methods.R SparseArray-misc-methods.R
        SparseMatrix-mult.R matrixStats-methods.R rowsum-methods.R
        randomSparseArray.R readSparseCSV.R zzz.R
git_url: https://git.bioconductor.org/packages/SparseArray
git_branch: RELEASE_3_18
git_last_commit: fe2f5b7
git_last_commit_date: 2023-11-06
Date/Publication: 2023-11-07
NeedsCompilation: yes
Packaged: 2023-11-07 23:43:49 UTC; biocbuild
Author: Hervé Pagès [aut, cre],
  Vince Carey [fnd],
  Rafael A. Irizarry [fnd]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
