Package: SparseArray
Title: High-performance sparse data representation and manipulation in
        R
Description: The SparseArray package provides array-like containers for
	efficient in-memory representation of multidimensional sparse data in
	R (arrays and matrices). The package defines the SparseArray virtual
	class and two concrete subclasses: COO_SparseArray and SVT_SparseArray.
	Each subclass uses its own internal representation of the nonzero
	multidimensional data: the "COO layout" and the "SVT layout",
	respectively. SVT_SparseArray objects mimic as much as possible the
	behavior of ordinary matrix and array objects in base R. In particular,
	they suppport most of the "standard matrix and array API" defined in
	base R and in the matrixStats package from CRAN.
biocViews: Infrastructure, DataRepresentation
URL: https://bioconductor.org/packages/SparseArray
BugReports: https://github.com/Bioconductor/SparseArray/issues
Version: 1.13.2
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com",
		comment=c(ORCID="0009-0002-8272-4522")),
	person("Vince", "Carey", role="fnd",
		email="stvjc@channing.harvard.edu",
		comment=c(ORCID="0000-0003-4046-0063")),
	person("Rafael A.", "Irizarry", role="fnd",
		email="rafa@ds.harvard.edu",
		comment=c(ORCID="0000-0002-3944-4309")),
	person("Jacques", "Serizay", role="ctb",
		comment=c(ORCID="0000-0002-4295-0624")))
Depends: R (>= 4.3.0), methods, Matrix, BiocGenerics (>= 0.43.1),
        MatrixGenerics (>= 1.11.1), S4Vectors (>= 0.43.2), S4Arrays (>=
        1.11.1)
Imports: utils, stats, matrixStats, IRanges, XVector
LinkingTo: S4Vectors, IRanges, XVector
Suggests: HDF5Array, ExperimentHub, testthat, knitr, rmarkdown,
        BiocStyle
VignetteBuilder: knitr
Collate: utils.R options.R OPBufTree.R thread-control.R
        sparseMatrix-utils.R is_nonzero.R SparseArray-class.R
        COO_SparseArray-class.R SVT_SparseArray-class.R
        extract_sparse_array.R read_block_as_sparse.R
        SparseArray-dim-tuning.R SparseArray-aperm.R
        SparseArray-subsetting.R SparseArray-subassignment.R
        SparseArray-abind.R SparseArray-summarization.R
        SparseArray-Arith-methods.R SparseArray-Compare-methods.R
        SparseArray-Logic-methods.R SparseArray-Math-methods.R
        SparseArray-Complex-methods.R SparseArray-misc-methods.R
        SparseArray-matrixStats.R rowsum-methods.R SparseMatrix-mult.R
        randomSparseArray.R readSparseCSV.R is_nonna.R NaArray-class.R
        NaArray-aperm.R NaArray-subsetting.R NaArray-subassignment.R
        NaArray-abind.R NaArray-summarization.R NaArray-Arith-methods.R
        NaArray-Compare-methods.R NaArray-Logic-methods.R
        NaArray-Math-methods.R NaArray-misc-methods.R
        NaArray-matrixStats.R zzz.R
git_url: https://git.bioconductor.org/packages/SparseArray
git_branch: devel
git_last_commit: 22b3b40
git_last_commit_date: 2026-05-01
Repository: Bioconductor 3.24
Date/Publication: 2026-05-01
NeedsCompilation: yes
Packaged: 2026-05-02 00:19:22 UTC; biocbuild
Author: Hervé Pagès [aut, cre] (ORCID: <https://orcid.org/0009-0002-8272-4522>),
  Vince Carey [fnd] (ORCID: <https://orcid.org/0000-0003-4046-0063>),
  Rafael A. Irizarry [fnd] (ORCID:
    <https://orcid.org/0000-0002-3944-4309>),
  Jacques Serizay [ctb] (ORCID: <https://orcid.org/0000-0002-4295-0624>)
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
