[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**SparseArray** is an R/Bioconductor package that provides a convenient array-like container for efficient in-memory representation and manipulation of multidimensional sparse data in R.

See https://bioconductor.org/packages/SparseArray for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

